{% extends "serviceform/base.html" %}
{% load i18n%}
{% block title %}{{service_form.name}}{% endblock %}
{% block serviceform-title %}{{service_form.name}}{% endblock %}
{% block content %}{%endblock %}
