{% load i18n serviceform_tags %}
<nav class="navbar navbar-default">
  <div class="container-fluid">
    <!-- Brand and toggle get grouped for better mobile display -->
    <div class="navbar-header">
      <button type="button" class="navbar-toggle collapsed"
              data-toggle="collapse"
              data-target="#bs-example-navbar-collapse-1"
              aria-expanded="false">
        <span class="sr-only">{% trans "Toggle navigation" %}</span>
        <span class="icon-bar"></span>
        <span class="icon-bar"></span>
        <span class="icon-bar"></span>
      </button>
      {% block navbar-brand %}
        <a class="navbar-brand" href="#"><span class="fa fa-user"></span></a>
      {% endblock %}
    </div>

    <!-- Collect the nav links, forms, and other content for toggling -->
    {% menu_items navbar_name as m_items %}
    {% block navbar-left %}
      <div class="collapse navbar-collapse" id="bs-example-navbar-collapse-1">
      <ul class="nav navbar-nav">
        {% for page in m_items.left %}
          {% if page.is_active %}
            <li class="active"><a href="{{ page.url }}">{{ page.title }} <span
                    class="sr-only">({% trans "current" %})</span></a></li>
          {% else %}
            <li><a href="{{ page.url }}">{{ page.title }}</a></li>
          {% endif %}
        {% endfor %}
      </ul>
    {% endblock %}

    {% block navbar-right %}
      <ul class="nav navbar-nav navbar-right">
        {% for page in m_items.right %}
          {% if page.is_active %}
            <li class="active"><a href="{{ page.url }}">{{ page.title }} <span
                    class="sr-only">({% trans "current" %})</span></a></li>
          {% else %}
            <li><a href="{{ page.url }}">{{ page.title }}</a></li>
          {% endif %}
        {% endfor %}
      </ul>
    {% endblock %}
    </div><!-- /.navbar-collapse -->
  </div><!-- /.container-fluid -->
</nav>