{% load serviceform_tags %}
{% load i18n %}
{% if form.errors %}
  <div class="row activity-row form-error">
    <div class="col-xs-7 col-xs-offset-1">
      {% trans "Please fix errors below" %}
    </div>
  </div>
{% endif %}

{% if category %}
  {% include "serviceform/participation/participation_form/snippets/level2categories.html" with level1cat=category%}
{% else %}
{% for level1cat in form.sub_items %}
  {% include "serviceform/participation/participation_form/snippets/level2categories.html" %}
{% endfor %}
{%endif %}
