{% extends "serviceform/participation/login_base.html" %}
{% load i18n crispy_forms_tags %}
{% block login-content %}
  {% if service_form.password %}
  <p>{% trans "If this is the first time you fill this form, you need a password to continue" %}.</p>
  {% endif %}
  {% if service_form.login_text %}
    <p>{{ service_form.login_text }}</p>
  {% endif %}
  {% crispy password_form %}
{% endblock %}