{% extends "serviceform/participation/login_base.html" %}
{% load i18n %}
{% block content %}
{% trans "You won't be getting any emails (as responsible for "%}{{service_form}}{% trans ") from this system any more."%}
{% endblock %}