{% load i18n serviceform_tags %}
  <h2>{% trans "Answers to questions" %}</h2>
  {% for q in service_form.questions %}
    <div class="report-question-title">{{ q.id_display }} {{ q.question }}</div>
    <ul>
      {% questionanswers q as qa_items %}
      {% for pq in qa_items %}
        {% if pq.answer %}
                 {% all_revisions as ar%}
          <li>{% if ar %}
          {{pq.participant.form_revision}}
          {% endif %}
          <a href="{% url "view_user" pq.participant.pk %}">{{ pq.cached_participant }}</a>:
          {% if pq.question.answer_type == 'boolean' %}
            {% if pq.answer %}
              {% trans "Yes" %}
            {% else %}
              {% trans "No" %}
            {% endif %}
          {% else %}
            {{ pq.answer|linebreaksbr }}
          {% endif %}
        {% endif %}
      {% endfor %}
    </ul>
  {% endfor %}