<div class="form-group">
  <label for="SRV_QUESTION_{{ question.pk }}"
         class="control-label questions-label-column {% if question.required %} requiredField{% endif %}">
    {{ question.question }}
    {% if question.required %} <span class="asteriskField">*</span>{% endif %}
  </label>
  <div class="checkbox questions-field-column controls question-checkbox checkbox-primary">
    <input class="checkbox question-checkbox form-control" id="SRV_QUESTION_{{ question.pk }}"
           name="SRV_QUESTION_{{ question.pk }}" type="checkbox" {% if question.answer%}checked{%endif%}/>
    <label></label>
  </div>
</div>