{% load i18n %}
  <h2>{% trans "All participants" %} ({{ps|length}})</h2>
  {% for p in ps %}
    {% include "serviceform/reports/snippets/_participant_row.html" with participant=p %}
  {% endfor %}


