{% extends "serviceform/participation/participation_flow_base.html" %}
{% load i18n crispy_forms_tags %}

{%block content %}
{{ block.super}}
<div class="helptext">{% trans "Are you sure you want to delete your participation permanently from the system?"%}</div>
{% crispy form %}
{% endblock%}