{% extends "serviceform/participation/participation_flow_base.html" %}
{% load i18n crispy_forms_tags %}
{%block content %}
{{ block.super}}
<div class="helptext">
{% if participant.is_updating %}
<li>  {% trans "You can delete your participation from the system by clicking"%} <a href="{% url "delete_participation"%}">{% trans "here"%}</a>.
<li>{% trans "Otherwise, please check if the information is correct and update it if necessary. " %}
{% else %}
  {% trans "Please fill in your contact details"%}
{% endif %}
</div>
{% crispy form %}
{% endblock%}