{% extends "serviceform/participation/participation_flow_base.html" %}
{% load i18n %}

{%block content %}
{{ block.super}}
<div class="helptext">{% trans "Email sent to"%} {{participant.email}}.
{%trans "Please verify your email address by clicking the link that is given in email. Then you can continue filling the form."%}</div>
{% endblock%}