# -*- coding: utf-8 -*-
# (c) 2017 Tuomas Airaksinen
#
# This file is part of Serviceform.
#
# Serviceform is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Serviceform is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Serviceform.  If not, see <http://www.gnu.org/licenses/>.

from .email import EmailMessage, EmailTemplate
from .participation import (ParticipationActivity, ParticipationActivityChoice, ParticipantLog,
                            QuestionAnswer)
from .people import Participant, ResponsibilityPerson
from .serviceform import (ServiceForm, FormRevision, Activity, ActivityChoice, Level1Category,
                          Level2Category, Question, ColorField)


from .mixins import (ContactDetailsMixinEmail, ContactDetailsMixin, CopyMixin, NameDescriptionMixin,
                     PasswordMixin, SubitemMixin)